@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -35,26 +35,30 @@ def main():
     CITATION = """
 Alonge, Michael, et al. "RaGOO: fast and accurate reference-guided scaffolding of draft genomes."
 Genome biology 20.1 (2019): 1-17.
+https://doi.org/10.1186/s13059-019-1829-6
     """
 
     description = """
-RagTag: Reference-guided scaffolding and misassembly correction.
+RagTag: Tools for fast and flexible genome assembly scaffolding and improvement.
 Version: %s
 
 usage: ragtag.py <command> [options]
     
     assembly improvement:
-      correct         misassembly correction 
+      correct         misassembly correction
       scaffold        synteny scaffolding
       merge           scaffold merging
+      patch           continuous scaffolding & gap filling
       
     file utilities:
-      agp2fasta       build a FASTA file from an AGP file
+      agp2fa          build a FASTA file from an AGP file
       agpcheck        check for valid AGP file format
-      updategff       update gff intervals
       asmstats        assembly statistics
+      splitasm        split an assembly at gaps
+      delta2paf       delta to PAF file conversion
+      paf2delta       PAF to delta file conversion
+      updategff       update gff intervals
       
-    
 
     options:
       -c, --citation  
@@ -88,8 +92,12 @@ usage: ragtag.py <command> [options]
             subcmd = ["ragtag_merge.py"] + sys.argv[2:]
             subprocess.call(subcmd)
 
-        elif cmd == "agp2fasta":
-            subcmd = ["ragtag_agp2fasta.py"] + sys.argv[2:]
+        elif cmd == "patch":
+            subcmd = ["ragtag_patch.py"] + sys.argv[2:]
+            subprocess.call(subcmd)
+
+        elif cmd == "agp2fa":
+            subcmd = ["ragtag_agp2fa.py"] + sys.argv[2:]
             subprocess.call(subcmd)
 
         elif cmd == "agpcheck":
@@ -104,6 +112,18 @@ usage: ragtag.py <command> [options]
             subcmd = ["ragtag_asmstats.py"] + sys.argv[2:]
             subprocess.call(subcmd)
 
+        elif cmd == "splitasm":
+            subcmd = ["ragtag_splitasm.py"] + sys.argv[2:]
+            subprocess.call(subcmd)
+
+        elif cmd == "delta2paf":
+            subcmd = ["ragtag_delta2paf.py"] + sys.argv[2:]
+            subprocess.call(subcmd)
+
+        elif cmd == "paf2delta":
+            subcmd = ["ragtag_paf2delta.py"] + sys.argv[2:]
+            subprocess.call(subcmd)
+
         else:
             print(description)
             print("\n** unrecognized command: %s **" % cmd)
