@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -39,6 +39,7 @@ from ragtag_utilities.AlignmentReader import PAFReader
 from ragtag_utilities.ContigAlignment import ContigAlignment
 from ragtag_utilities.AGPFile import AGPFile
 from ragtag_utilities.Aligner import Minimap2Aligner
+from ragtag_utilities.Aligner import UnimapAligner
 from ragtag_utilities.Aligner import Minimap2SAMAligner
 from ragtag_utilities.Aligner import NucmerAligner
 
@@ -89,23 +90,23 @@ def read_genome_alignments(aln_file, query_blacklist, ref_blacklist):
 
 def get_median_read_coverage(output_path, num_threads, overwrite_files):
     """ Given the read alignments, use samtools stats to return an approximate median coverage value. """
-    log("Calculating global read coverage")
-    if os.path.isfile(output_path + "c_reads_against_query.s.bam.stats"):
+    log("INFO", "Calculating global read coverage")
+    if os.path.isfile(output_path + "ragtag.correct.reads.s.bam.stats"):
         if not overwrite_files:
-            log("retaining pre-existing file: " + output_path + "c_reads_against_query.s.bam.stats")
+            log("INFO", "Retaining pre-existing file: " + output_path + "ragtag.correct.reads.s.bam.stats")
         else:
-            log("overwriting pre-existing file: " + output_path + "c_reads_against_query.s.bam.stats")
-            st = pysam.stats("-@", str(num_threads), output_path + "c_reads_against_query.s.bam")
-            with open(output_path + "c_reads_against_query.s.bam.stats", "w") as f:
+            log("INFO", "Overwriting pre-existing file: " + output_path + "ragtag.correct.reads.s.bam.stats")
+            st = pysam.stats("-@", str(num_threads), output_path + "ragtag.correct.reads.s.bam")
+            with open(output_path + "ragtag.correct.reads.s.bam.stats", "w") as f:
                 f.write(st)
     else:
-        st = pysam.stats("-@", str(num_threads), output_path + "c_reads_against_query.s.bam")
-        with open(output_path + "c_reads_against_query.s.bam.stats", "w") as f:
+        st = pysam.stats("-@", str(num_threads), output_path + "ragtag.correct.reads.s.bam")
+        with open(output_path + "ragtag.correct.reads.s.bam.stats", "w") as f:
             f.write(st)
 
     # Get the coverage histogram (for 1 to 1k)
     covs = []
-    with open(output_path + "c_reads_against_query.s.bam.stats") as f:
+    with open(output_path + "ragtag.correct.reads.s.bam.stats") as f:
         for line in f:
             if line.startswith("COV"):
                 covs.append(int(line.split("\t")[3]))
@@ -126,26 +127,26 @@ def get_median_read_coverage(output_path, num_threads, overwrite_files):
 
 def run_samtools(output_path, num_threads, overwrite_files):
     """ Compress, sort and index alignments with pysam. """
-    if os.path.isfile(output_path + "c_reads_against_query.s.bam"):
+    if os.path.isfile(output_path + "ragtag.correct.reads.s.bam"):
         if not overwrite_files:
-            log("Retaining pre-existing file: " + output_path + "c_reads_against_query.s.bam")
+            log("INFO", "Retaining pre-existing file: " + output_path + "ragtag.correct.reads.s.bam")
         else:
-            log("Overwriting pre-existing file: " + output_path + "c_reads_against_query.s.bam")
-            pysam.view("-@", str(num_threads), "-b", "-o", output_path + "c_reads_against_query.bam", output_path + "c_reads_against_query.sam", catch_stdout=False)
-            pysam.sort("-@", str(num_threads), "-o", output_path + "c_reads_against_query.s.bam", output_path + "c_reads_against_query.bam", catch_stdout=False)
+            log("INFO", "Overwriting pre-existing file: " + output_path + "ragtag.correct.reads.s.bam")
+            pysam.view("-@", str(num_threads), "-b", "-o", output_path + "ragtag.correct.reads.bam", output_path + "ragtag.correct.reads.sam", catch_stdout=False)
+            pysam.sort("-@", str(num_threads), "-o", output_path + "ragtag.correct.reads.s.bam", output_path + "ragtag.correct.reads.bam", catch_stdout=False)
     else:
-        pysam.view("-@", str(num_threads), "-b", "-o", output_path + "c_reads_against_query.bam", output_path + "c_reads_against_query.sam", catch_stdout=False)
-        pysam.sort("-@", str(num_threads), "-o", output_path + "c_reads_against_query.s.bam", output_path + "c_reads_against_query.bam", catch_stdout=False)
+        pysam.view("-@", str(num_threads), "-b", "-o", output_path + "ragtag.correct.reads.bam", output_path + "ragtag.correct.reads.sam", catch_stdout=False)
+        pysam.sort("-@", str(num_threads), "-o", output_path + "ragtag.correct.reads.s.bam", output_path + "ragtag.correct.reads.bam", catch_stdout=False)
 
-    log("Indexing read alignments")
-    if os.path.isfile(output_path + "c_reads_against_query.s.bam.bai"):
+    log("INFO", "Indexing read alignments")
+    if os.path.isfile(output_path + "ragtag.correct.reads.s.bam.bai"):
         if not overwrite_files:
-            log("Retaining pre-existing file: " + output_path + "c_reads_against_query.s.bam.bai")
+            log("INFO", "Retaining pre-existing file: " + output_path + "ragtag.correct.reads.s.bam.bai")
         else:
-            log("Overwriting pre-existing file: " + output_path + "c_reads_against_query.s.bam.bai")
-            pysam.index(output_path + "c_reads_against_query.s.bam", catch_stdout=False)
+            log("INFO", "Overwriting pre-existing file: " + output_path + "ragtag.correct.reads.s.bam.bai")
+            pysam.index(output_path + "ragtag.correct.reads.s.bam", catch_stdout=False)
     else:
-        pysam.index(output_path + "c_reads_against_query.s.bam", catch_stdout=False)
+        pysam.index(output_path + "ragtag.correct.reads.s.bam", catch_stdout=False)
 
 
 def clean_breaks(val_breaks, d):
@@ -161,7 +162,7 @@ def clean_breaks(val_breaks, d):
     return breaks
 
 
-def validate_breaks(ctg_breaks, output_path, num_threads, overwrite_files, min_break_end_dist, max_cutoff, min_cutoff, window_size=10000, num_devs=3, clean_dist=1000, debug=False):
+def validate_breaks(ctg_breaks, output_path, num_threads, overwrite_files, min_break_end_dist, max_cutoff, min_cutoff, window_size=10000, num_devs=5, clean_dist=1000, debug=False):
     """
     """
     # Get the median coverage over all bp
@@ -169,16 +170,16 @@ def validate_breaks(ctg_breaks, output_path, num_threads, overwrite_files, min_b
     dev = round(math.sqrt(glob_med))
 
     if max_cutoff == -1:
-        max_cutoff = glob_med + (num_devs*dev)
+        max_cutoff = glob_med*12
 
     if min_cutoff == -1:
-        min_cutoff = max(0, (glob_med - (num_devs*dev)))
+        min_cutoff = max(5, (glob_med - (num_devs*dev)))
 
-    log("The global median read coverage is %dX" % glob_med)
-    log("The max and min coverage thresholds are %dX and %dX, respectively" % (max_cutoff, min_cutoff))
+    log("INFO", "The global median read coverage is %dX" % glob_med)
+    log("INFO", "The max and min coverage thresholds are %dX and %dX, respectively" % (max_cutoff, min_cutoff))
 
     # Go through each break point and query the coverage within the vicinity of the breakpoint.
-    bam = pysam.AlignmentFile(output_path + "c_reads_against_query.s.bam")
+    bam = pysam.AlignmentFile(output_path + "ragtag.correct.reads.s.bam")
     validated_ctg_breaks = dict()
     for ctg in ctg_breaks:
         val_breaks = []
@@ -193,7 +194,7 @@ def validate_breaks(ctg_breaks, output_path, num_threads, overwrite_files, min_b
                 continue
 
             region = "%s:%d-%d" % (ctg, min_range, max_range-1)
-            depth_out = pysam.samtools.depth("-aa", "-r", region, output_path + "c_reads_against_query.s.bam")
+            depth_out = pysam.samtools.depth("-aa", "-r", region, output_path + "ragtag.correct.reads.s.bam")
             covs = np.asarray(
                 [j.split("\t")[2] for j in [i for i in depth_out.rstrip().split("\n")]],
                 dtype=np.int32
@@ -220,7 +221,7 @@ def validate_breaks(ctg_breaks, output_path, num_threads, overwrite_files, min_b
                 status = "high cov"
 
             if debug:
-                log("query: %s, original break: %s, window start: %d, window end: %d, status: %s, new_break: %s, cov max: %d, cov min: %d" %(ctg, b, min_range, max_range, status, str(new_break), cov_max, cov_min))
+                log("DEBUG", "query: %s, original break: %s, window start: %d, window end: %d, status: %s, new_break: %s, cov max: %d, cov min: %d" % (ctg, b, min_range, max_range, status, str(new_break), cov_max, cov_min))
 
         validated_ctg_breaks[ctg] = clean_breaks(val_breaks, clean_dist)
 
@@ -241,8 +242,8 @@ def make_gff_interval_tree(gff_file):
                 assert start < end
 
                 if end - start > 100000:
-                    coords = "%s:%d-%d" %(h, start+1, end)
-                    log("WARNING: large interval in this gff file (%s). This could disproportionately invalidate putative query breakpoints." % coords)
+                    coords = "%s:%d-%d" % (h, start+1, end)
+                    log("WARNING", "Large interval in this gff file (%s). This could disproportionately invalidate putative query breakpoints." % coords)
                 t[h][start:end] = (start, end)
 
     return t
@@ -253,11 +254,11 @@ def write_breaks(out_file, query_file, ctg_breaks, overwrite, remove_suffix):
     # Check if the output file already exists
     if os.path.isfile(out_file):
         if not overwrite:
-            log("Retaining pre-existing file: " + out_file)
+            log("INFO", "Retaining pre-existing file: " + out_file)
             return
 
         else:
-            log("Overwriting pre-existing file: " + out_file)
+            log("INFO", "Overwriting pre-existing file: " + out_file)
 
     fai = pysam.FastaFile(query_file)
     all_q_seqs = sorted(fai.references)
@@ -274,7 +275,7 @@ def write_breaks(out_file, query_file, ctg_breaks, overwrite, remove_suffix):
             # Add suffix to query header, unless otherwise requested
             unchanged_comp_header = q
             if not remove_suffix:
-                unchanged_comp_header = q + ":0" + "-" + str(fai.get_reference_length(q)) + "(+)"
+                unchanged_comp_header = q + "_1" + "_" + str(fai.get_reference_length(q)) + "_+"
 
             agp.add_seq_line(
                     q,
@@ -298,7 +299,7 @@ def write_breaks(out_file, query_file, ctg_breaks, overwrite, remove_suffix):
                         str(i),
                         str(pid),
                         "W",
-                        q + ":" + str(start) + "-" + str(i) + "(+)",
+                        q + "_" + str(start+1) + "_" + str(i) + "_+",
                         "1",
                         str(i-start),
                         "+"
@@ -313,19 +314,21 @@ def write_breaks(out_file, query_file, ctg_breaks, overwrite, remove_suffix):
                     str(fai.get_reference_length(q)),
                     str(pid),
                     "W",
-                    q + ":" + str(start) + "-" + str(fai.get_reference_length(q)) + "(+)",
+                    q + "_" + str(start+1) + "_" + str(fai.get_reference_length(q)) + "_+",
                     "1",
                     str(fai.get_reference_length(q)-start),
                     "+"
             )
 
-    log("Writing: " + out_file)
+    log("INFO", "Writing: " + out_file)
     agp.write()
     fai.close()
 
 
 def main():
-    parser = argparse.ArgumentParser(description='Reference-guided misassembly correction', usage="ragtag.py correct <reference.fa> <query.fa>")
+    description = "Homology-based assembly correction: Correct sequences in 'query.fa' by comparing them to " \
+                  "sequences in 'reference.fa'>"
+    parser = argparse.ArgumentParser(description=description, usage="ragtag.py correct <reference.fa> <query.fa>")
 
     parser.add_argument("reference", metavar="<reference.fa>", nargs='?', default="", type=str, help="reference fasta file (uncompressed or bgzipped)")
     parser.add_argument("query", metavar="<query.fa>", nargs='?', default="", type=str, help="query fasta file (uncompressed or bgzipped)")
@@ -334,7 +337,7 @@ def main():
     cor_options.add_argument("-f", metavar="INT", type=int, default=1000, help="minimum unique alignment length [1000]")
     cor_options.add_argument("--remove-small", action="store_true", default=False, help="remove unique alignments shorter than -f")
     cor_options.add_argument("-q", metavar="INT", type=int, default=10, help="minimum mapq (NA for Nucmer alignments) [10]")
-    cor_options.add_argument("-d", metavar="INT", type=int, default=100000, help="alignment merge distance [100000]")
+    cor_options.add_argument("-d", metavar="INT", type=int, default=100000, help="maximum alignment merge distance [100000]")
     cor_options.add_argument("-b", metavar="INT", type=int, default=5000, help="minimum break distance from contig ends [5000]")
     cor_options.add_argument("-e", metavar="<exclude.txt>", type=str, default="", help="list of reference headers to ignore [null]")
     cor_options.add_argument("-j", metavar="<skip.txt>", type=str, default="", help="list of query headers to leave uncorrected [null]")
@@ -350,16 +353,17 @@ def main():
 
     aln_options = parser.add_argument_group("mapping options")
     mm2_default = "-x asm5"
-    aln_options.add_argument("-t", metavar="INT", type=int, default=1, help="number of minimap2 threads [1]")
-    aln_options.add_argument("--aligner", metavar="PATH", type=str, default="minimap2", help="whole genome aligner executable ('nucmer' or 'minimap2') [minimap2]")
+    aln_options.add_argument("-t", metavar="INT", type=int, default=1, help="number of minimap2/unimap threads [1]")
+    aln_options.add_argument("--aligner", metavar="PATH", type=str, default="minimap2", help="whole genome aligner executable ('nucmer', 'unimap' or 'minimap2') [minimap2]")
     aln_options.add_argument("--mm2-params", metavar="STR", type=str, default=mm2_default, help="space delimited minimap2 whole genome alignment parameters ['%s']" % mm2_default)
-    aln_options.add_argument("--nucmer-params", metavar="STR", type=str, default="-l 100 -c 500", help="space delimted nucmer whole genome alignment parameters ['-l 100 -c 500']")
+    aln_options.add_argument("--unimap-params", metavar="STR", type=str, default=mm2_default, help="space delimited unimap parameters ['%s']" % mm2_default)
+    aln_options.add_argument("--nucmer-params", metavar="STR", type=str, default="--maxmatch -l 100 -c 500", help="space delimted nucmer whole genome alignment parameters ['--maxmatch -l 100 -c 500']")
 
     val_options = parser.add_argument_group("validation options")
     val_options.add_argument("--read-aligner", metavar="PATH", type=str, default="minimap2", help="read aligner executable (only 'minimap2' is allowed) [minimap2]")
     val_options.add_argument("-R", metavar="<reads.fasta>", type=str, default="", help="validation reads (uncompressed or gzipped) [null]")
     val_options.add_argument("-F", metavar="<reads.fofn>", type=str, default="", help="same as '-R', but a list of files [null]")
-    val_options.add_argument("-T", metavar="STR", type=str, default="", help="read type. 'sr' and 'corr' accepted for short reads and error corrected long-reads, respectively [null]")
+    val_options.add_argument("-T", metavar="STR", type=str, default="", help="read type. 'sr', 'ont' and 'corr' accepted for Illumina, nanopore and error corrected long-reads, respectively [null]")
     val_options.add_argument("-v", metavar="INT", type=int, default=10000, help="coverage validation window size [10000]")
     val_options.add_argument("--max-cov", metavar="INT", type=int, default=-1, help="break sequences at regions at or above this coverage level [AUTO]")
     val_options.add_argument("--min-cov", metavar="INT", type=int, default=-1, help="break sequences at regions at or below this coverage level [AUTO]")
@@ -369,11 +373,10 @@ def main():
 
     if not args.reference or not args.query:
         parser.print_help()
-        print("\n** The reference and query FASTA files are required **")
-        sys.exit()
+        sys.exit("\n** The reference and query FASTA files are required **")
 
-    log("RagTag " + get_ragtag_version())
-    log("CMD: ragtag.py correct " + " ".join(sys.argv[1:]))
+    log("VERSION", "RagTag " + get_ragtag_version())
+    log("CMD", "ragtag.py correct " + " ".join(sys.argv[1:]))
 
     reference_file = os.path.abspath(args.reference)
     query_file = os.path.abspath(args.query)
@@ -398,15 +401,16 @@ def main():
     if not os.path.isdir(output_path):
         os.mkdir(output_path)
     output_path = os.path.abspath(output_path) + "/"
+    file_prefix = "ragtag.correct"
 
     # Setup a log file for external RagTag scripts
-    ragtag_log = output_path + "ragtag.correct.err"
+    ragtag_log = output_path + file_prefix + ".err"
     open(ragtag_log, "w").close()  # Wipe the log file
 
     overwrite_files = args.w
     remove_suffix = not args.u
     if remove_suffix:
-        log("WARNING: Without '-u' invoked, some component/object AGP pairs might share the same ID. Some external programs/databases don't like this. To ensure valid AGP format, use '-u'.")
+        log("WARNING", "Without '-u' invoked, some component/object AGP pairs might share the same ID. Some external programs/databases don't like this. To ensure valid AGP format, use '-u'.")
 
     gff_file = args.gff
     if gff_file:
@@ -432,10 +436,11 @@ def main():
     # Get aligner arguments
     genome_aligner_path = args.aligner
     genome_aligner = genome_aligner_path.split("/")[-1]
-    if genome_aligner.split("/")[-1] not in {'minimap2', 'nucmer'}:
-        raise ValueError("Must specify either 'minimap2' or 'nucmer' (PATHs allowed) with '--aligner'.")
+    if genome_aligner.split("/")[-1] not in {'minimap2', 'unimap', 'nucmer'}:
+        raise ValueError("Must specify either 'minimap2', 'unimap', or 'nucmer' (PATHs allowed) with '--aligner'.")
 
     mm2_params = args.mm2_params
+    unimap_params = args.unimap_params
     nucmer_params = args.nucmer_params
 
     # Mapq filtering params
@@ -446,6 +451,8 @@ def main():
     # Add the number of mm2 threads if the mm2 params haven't been overridden.
     if mm2_params == mm2_default:
         mm2_params += " -t " + str(num_threads)
+    if unimap_params == mm2_default:
+        unimap_params += " -t " + str(num_threads)
 
     # Check if intra/inter breaking is desired
     break_intra = True
@@ -463,7 +470,7 @@ def main():
     # read-alignment parameters
     val_reads = args.R
     val_reads_fofn = args.F
-    val_reads_tech = args.T
+    val_reads_tech = args.T.upper()
     read_aligner_path = args.read_aligner
     read_aligner = read_aligner_path.split("/")[-1]
     if read_aligner != "minimap2":
@@ -501,28 +508,30 @@ def main():
 
     # Debugging options
     debug_mode = args.debug
-    debug_non_fltrd_file = output_path + "ragtag.correction.debug.unfiltered.paf"
-    debug_fltrd_file = output_path + "ragtag.correction.debug.filtered.paf"
-    debug_merged_file = output_path + "ragtag.correction.debug.merged.paf"
-    debug_query_info_file = output_path + "ragtag.correction.debug.query.info.txt"
+    debug_non_fltrd_file = output_path + file_prefix + ".debug.unfiltered.paf"
+    debug_fltrd_file = output_path + file_prefix + ".debug.filtered.paf"
+    debug_merged_file = output_path + file_prefix + ".debug.merged.paf"
+    debug_query_info_file = output_path + file_prefix + ".debug.query.info.txt"
 
     # Align the query to the reference.
-    log("Mapping the query genome to the reference genome")
+    log("INFO", "Mapping the query genome to the reference genome")
     if genome_aligner == "minimap2":
-        al = Minimap2Aligner(reference_file, [query_file], genome_aligner_path, mm2_params, output_path + "c_query_against_ref", in_overwrite=overwrite_files)
+        al = Minimap2Aligner(reference_file, [query_file], genome_aligner_path, mm2_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
+    elif genome_aligner == "unimap":
+        al = UnimapAligner(reference_file, [query_file], genome_aligner_path, unimap_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
     else:
-        al = NucmerAligner(reference_file, [query_file], genome_aligner_path, nucmer_params, output_path + "c_query_against_ref", in_overwrite=overwrite_files)
+        al = NucmerAligner(reference_file, [query_file], genome_aligner_path, nucmer_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
     al.run_aligner()
 
     # If alignments are from Nucmer, convert from delta to paf.
     if genome_aligner == "nucmer":
-        cmd = ["ragtag_delta2paf.py", output_path + "c_query_against_ref.delta"]
-        run_oae(cmd, output_path + "c_query_against_ref.paf", ragtag_log)
+        cmd = ["ragtag_delta2paf.py", output_path + file_prefix + ".asm.delta"]
+        run_oae(cmd, output_path + file_prefix + ".asm.paf", ragtag_log)
 
     # Read and organize the alignments.
-    log('Reading whole genome alignments')
+    log("INFO", "Reading whole genome alignments")
     # ctg_alns = dict :: key=query header, value=ContigAlignment object
-    ctg_alns = read_genome_alignments(output_path + "c_query_against_ref.paf", query_blacklist, ref_blacklist)
+    ctg_alns = read_genome_alignments(output_path + file_prefix + ".asm.paf", query_blacklist, ref_blacklist)
 
     # Filter and merge the alignments.
     if debug_mode:
@@ -532,7 +541,7 @@ def main():
         open(debug_merged_file, "w").close()
         open(debug_query_info_file, "w").close()
 
-    log("Filtering and merging alignments")
+    log("INFO", "Filtering and merging alignments")
     for i in ctg_alns:
 
         # Write unfiltered alignments
@@ -550,7 +559,7 @@ def main():
                     with open(debug_fltrd_file, "a") as f:
                         f.write(str(ctg_alns[i]))
 
-                ctg_alns[i] = ctg_alns[i].merge_alns(merge_dist=merge_dist)
+                ctg_alns[i] = ctg_alns[i].merge_alns(merge_dist=merge_dist, careful_merge=True)
 
     # Get the putative breakpoints for each query sequence, if any.
     ctg_breaks = dict()
@@ -582,33 +591,40 @@ def main():
 
     # If desired, validate the putative breakpoints by observing read coverage.
     if read_files:
-        log("Validating putative query breakpoints via read alignment.")
-        log("Aligning reads to query sequences.")
-        if not os.path.isfile(output_path + "c_reads_against_query.s.bam"):
-            if val_reads_tech == "sr":
+        log("INFO", "Validating putative query breakpoints via read alignment")
+        log("INFO", "Aligning reads to query sequences")
+        if not os.path.isfile(output_path + file_prefix + ".reads.s.bam"):
+            if val_reads_tech == "SR":
                 al = Minimap2SAMAligner(query_file, read_files, read_aligner_path, "-ax sr -t " + str(num_threads),
-                                        output_path + "c_reads_against_query", in_overwrite=overwrite_files)
-            elif val_reads_tech == "corr":
-                al = Minimap2SAMAligner(query_file, read_files, read_aligner_path, "-ax asm5 -t " + str(num_threads),
-                                        output_path + "c_reads_against_query", in_overwrite=overwrite_files)
+                                        output_path + file_prefix + ".reads", in_overwrite=overwrite_files)
+
+            elif val_reads_tech == "ONT":
+                al = Minimap2SAMAligner(query_file, read_files, read_aligner_path, "-ax map-ont -t " + str(num_threads),
+                                        output_path + file_prefix + ".reads", in_overwrite=overwrite_files)
+
+            elif val_reads_tech == "CORR":
+                al = Minimap2SAMAligner(query_file, read_files, read_aligner_path, "-ax asm20 -t " + str(num_threads),
+                                        output_path + file_prefix + ".reads", in_overwrite=overwrite_files)
             else:
-                raise ValueError("'-T' must be either 'sr' or 'corr'.")
+                raise ValueError("'-T' must be either 'sr', 'ont' or 'corr'.")
             al.run_aligner()
         else:
-            log("Retaining pre-existing read alignments: " + output_path + "c_reads_against_query.s.bam")
+            log("INFO", "Retaining pre-existing read alignments: " + output_path + file_prefix + ".reads.s.bam")
 
         # Compress, sort and index the alignments.
-        log("Compressing, sorting, and indexing read alignments")
+        log("INFO", "Compressing, sorting, and indexing read alignments")
         run_samtools(output_path, num_threads, overwrite_files)
 
         # Validate the breakpoints
-        log("Validating putative query breakpoints")
+        log("INFO", "Validating putative query breakpoints")
         
-        # Give at least 10k/1k from ctg ends for coverage to accumulate for corr and sr, respectively.
+        # Give at least 10k/1k from ctg ends for coverage to accumulate for long and short reads, respectively.
         val_min_break_end_dist = min_break_end_dist
-        if val_reads_tech == "corr":
+        if val_reads_tech == "CORR":
             val_min_break_end_dist = max(10000, min_break_end_dist)
-        if val_reads_tech == "sr":
+        if val_reads_tech == "ONT":
+            val_min_break_end_dist = max(10000, min_break_end_dist)
+        if val_reads_tech == "SR":
             val_min_break_end_dist = max(1000, min_break_end_dist)
             
         # Validate the breakpoints
@@ -616,14 +632,14 @@ def main():
 
     # Check if we need to avoid gff intervals
     if gff_file:
-        log("Avoiding breaks within GFF intervals")
+        log("INFO", "Avoiding breaks within GFF intervals")
         it = make_gff_interval_tree(gff_file)
         non_gff_breaks = dict()
         for ctg in ctg_breaks:
             new_breaks = []
             for i in ctg_breaks[ctg]:
                 if it[ctg][i]:
-                    log("Avoiding breaking %s at %d. This point intersects a feature in the gff file." % (ctg, i))
+                    log("INFO", "Avoiding breaking %s at %d. This point intersects a feature in the gff file." % (ctg, i))
                 else:
                     new_breaks.append(i)
             if new_breaks:
@@ -631,21 +647,19 @@ def main():
         ctg_breaks = non_gff_breaks
 
     # Write the summary of query sequence breaks in AGP format
-    agp_file = output_path + "ragtag.correction.agp"
+    agp_file = output_path + file_prefix + ".agp"
     write_breaks(agp_file, query_file, ctg_breaks, True, remove_suffix)
 
     # Write the scaffolds.
-    log("Writing broken contigs")
-    qf_name = query_file.split("/")[-1]
-    qf_pref = qf_name[:qf_name.rfind(".")]
+    log("INFO", "Writing broken contigs")
     cmd = [
         "ragtag_break_query.py",
         agp_file,
         query_file
     ]
-    run_oae(cmd, output_path + qf_pref + ".corrected.fasta", ragtag_log)
+    run_oae(cmd, output_path + file_prefix + ".fasta", ragtag_log)
 
-    log("Goodbye")
+    log("INFO", "Goodbye")
 
 
 if __name__ == "__main__":
