@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -36,6 +36,7 @@ from ragtag_utilities.AlignmentReader import PAFReader
 from ragtag_utilities.ContigAlignment import ContigAlignment
 from ragtag_utilities.AGPFile import AGPFile
 from ragtag_utilities.Aligner import Minimap2Aligner
+from ragtag_utilities.Aligner import UnimapAligner
 from ragtag_utilities.Aligner import NucmerAligner
 
 
@@ -56,15 +57,15 @@ def remove_contained(a):
     return o
 
 
-def write_orderings(out_agp_file, out_confidence_file, query_file, ordering_dict, ctg_dict, gap_dict, gap_type_dict, make_chr0, overwrite, add_suffix):
+def write_orderings(out_agp_file, out_confidence_file, query_file, ordering_dict, ctg_dict, gap_dict, gap_type_dict, make_chr0, overwrite, add_suffix, skip_no_cat):
     # Check if the output file already exists
     if os.path.isfile(out_agp_file):
         if not overwrite:
-            log("Retaining pre-existing file: " + out_agp_file)
+            log("INFO", "Retaining pre-existing file: " + out_agp_file)
             return
 
         else:
-            log("Overwriting pre-existing file: " + out_agp_file)
+            log("INFO", "Overwriting pre-existing file: " + out_agp_file)
 
     # Proceed with writing the intermediate output
     placed_seqs = set()
@@ -137,44 +138,59 @@ def write_orderings(out_agp_file, out_confidence_file, query_file, ordering_dict
     unplaced_seqs = sorted(list(all_seqs - placed_seqs))
     if unplaced_seqs:
         if make_chr0:
-            pos = 0
-            pid = 1
-            new_ref_header = "Chr0_RagTag"
-            for q in unplaced_seqs:
-                out_agp_line = []
-                qlen = fai.get_reference_length(q)
-                out_agp_line.append(new_ref_header)
-                out_agp_line.append(str(pos+1))
-                pos += qlen
-                out_agp_line.append(str(pos))
-                out_agp_line.append(str(pid))
-                out_agp_line.append("W")
-                out_agp_line.append(q)
-                out_agp_line.append("1")
-                out_agp_line.append(str(qlen))
-                out_agp_line.append("+")
+            cat_seqs = all_seqs - placed_seqs - skip_no_cat
+            if cat_seqs:
+                pos = 0
+                pid = 1
+                new_ref_header = "Chr0_RagTag"
+                for q in sorted(list(cat_seqs)):
+                    out_agp_line = []
+                    qlen = fai.get_reference_length(q)
+                    out_agp_line.append(new_ref_header)
+                    out_agp_line.append(str(pos+1))
+                    pos += qlen
+                    out_agp_line.append(str(pos))
+                    out_agp_line.append(str(pid))
+                    out_agp_line.append("W")
+                    out_agp_line.append(q)
+                    out_agp_line.append("1")
+                    out_agp_line.append(str(qlen))
+                    out_agp_line.append("+")
 
-                agp.add_seq_line(*out_agp_line)
-                pid += 1
+                    agp.add_seq_line(*out_agp_line)
+                    pid += 1
 
-                # Now for the gap, since we are making a chr0
-                out_agp_line = []
-                out_agp_line.append(new_ref_header)
-                out_agp_line.append(str(pos+1))
-                pos += 100
-                out_agp_line.append(str(pos))
-                out_agp_line.append(str(pid))
-                out_agp_line.append("U")
-                out_agp_line.append("100")
-                out_agp_line.append("contig")
-                out_agp_line.append("no")
-                out_agp_line.append("na")
+                    # Now for the gap, since we are making a chr0
+                    out_agp_line = []
+                    out_agp_line.append(new_ref_header)
+                    out_agp_line.append(str(pos+1))
+                    pos += 100
+                    out_agp_line.append(str(pos))
+                    out_agp_line.append(str(pid))
+                    out_agp_line = out_agp_line + ["U", "100", "contig", "no", "na"]
+                    agp.add_gap_line(*out_agp_line)
+                    pid += 1
 
-                agp.add_gap_line(*out_agp_line)
-                pid += 1
-
-            # Remove the final unecessary gap
-            agp.pop_agp_line()
+                # Remove the final unnecessary gap
+                agp.pop_agp_line()
+            if skip_no_cat:
+                # List these unplaced contigs individually
+                for q in sorted(list(skip_no_cat)):
+                    out_agp_line = []
+                    qlen = fai.get_reference_length(q)
+                    if add_suffix:
+                        out_agp_line.append(q + "_RagTag")
+                    else:
+                        out_agp_line.append(q)
+                    out_agp_line.append("1")
+                    out_agp_line.append(str(qlen))
+                    out_agp_line.append("1")
+                    out_agp_line.append("W")
+                    out_agp_line.append(q)
+                    out_agp_line.append("1")
+                    out_agp_line.append(str(qlen))
+                    out_agp_line.append("+")
+                    agp.add_seq_line(*out_agp_line)
         else:
             # List the unplaced contigs individually
             for q in unplaced_seqs:
@@ -248,18 +264,21 @@ def read_genome_alignments(aln_file, query_blacklist, ref_blacklist):
 
 
 def main():
-    parser = argparse.ArgumentParser(description='Reference-guided scaffolding', usage="ragtag.py scaffold <reference.fa> <query.fa>")
+    description = "Homology-based assembly scaffolding: Order and orient sequences in 'query.fa' by comparing them to " \
+                  "sequences in 'reference.fa'>"
+    parser = argparse.ArgumentParser(description=description, usage="ragtag.py scaffold <reference.fa> <query.fa>")
 
     parser.add_argument("reference", metavar="<reference.fa>", nargs='?', default="", type=str, help="reference fasta file (uncompressed or bgzipped)")
     parser.add_argument("query", metavar="<query.fa>", nargs='?', default="", type=str, help="query fasta file (uncompressed or bgzipped)")
 
     scaf_options = parser.add_argument_group("scaffolding options")
-    scaf_options.add_argument("-e", metavar="<exclude.txt>", type=str, default="", help="list of reference headers to ignore [null]")
-    scaf_options.add_argument("-j", metavar="<skip.txt>", type=str, default="", help="list of query headers to leave unplaced [null]")
+    scaf_options.add_argument("-e", metavar="<exclude.txt>", type=str, default="", help="list of reference sequences to ignore [null]")
+    scaf_options.add_argument("-j", metavar="<skip.txt>", type=str, default="", help="list of query sequences to leave unplaced [null]")
+    scaf_options.add_argument("-J", metavar="<hard-skip.txt>", type=str, default="", help="list of query headers to leave unplaced and exclude from 'chr0' ('-C') [null]")
     scaf_options.add_argument("-f", metavar="INT", type=int, default=1000, help="minimum unique alignment length [1000]")
-    scaf_options.add_argument("--remove-small", action="store_true", default=False, help="remove unique alignments shorter than -f")
+    scaf_options.add_argument("--remove-small", action="store_true", default=False, help="remove unique alignments shorter than '-f'")
     scaf_options.add_argument("-q", metavar="INT", type=int, default=10, help="minimum mapq (NA for Nucmer alignments) [10]")
-    scaf_options.add_argument("-d", metavar="INT", type=int, default=100000, help="alignment merge distance [100000]")
+    scaf_options.add_argument("-d", metavar="INT", type=int, default=100000, help="maximum alignment merge distance [100000]")
     scaf_options.add_argument("-i", metavar="FLOAT", type=float, default=0.2, help="minimum grouping confidence score [0.2]")
     scaf_options.add_argument("-a", metavar="FLOAT", type=float, default=0.0, help="minimum location confidence score [0.0]")
     scaf_options.add_argument("-s", metavar="FLOAT", type=float, default=0.0, help="minimum orientation confidence score [0.0]")
@@ -275,20 +294,20 @@ def main():
     io_options.add_argument("--debug", action='store_true', default=False, help=argparse.SUPPRESS)
 
     aln_options = parser.add_argument_group("mapping options")
-    aln_options.add_argument("-t", metavar="INT", type=int, default=1, help="number of minimap2 threads [1]")
-    aln_options.add_argument("--aligner", metavar="PATH", type=str, default="minimap2", help="aligner executable ('nucmer' or 'minimap2') [minimap2]")
+    aln_options.add_argument("-t", metavar="INT", type=int, default=1, help="number of minimap2/unimap threads [1]")
+    aln_options.add_argument("--aligner", metavar="PATH", type=str, default="minimap2", help="aligner executable ('nucmer', 'unimap' or 'minimap2') [minimap2]")
     mm2_default = "-x asm5"
     aln_options.add_argument("--mm2-params", metavar="STR", type=str, default=mm2_default, help="space delimited minimap2 parameters ['%s']" % mm2_default)
-    aln_options.add_argument("--nucmer-params", metavar="STR", type=str, default="-l 100 -c 500", help="space delimted nucmer parameters ['-l 100 -c 500']")
+    aln_options.add_argument("--unimap-params", metavar="STR", type=str, default=mm2_default, help="space delimited unimap parameters ['%s']" % mm2_default)
+    aln_options.add_argument("--nucmer-params", metavar="STR", type=str, default="--maxmatch -l 100 -c 500", help="space delimted nucmer parameters ['--maxmatch -l 100 -c 500']")
 
     args = parser.parse_args()
     if not args.reference or not args.query:
         parser.print_help()
-        print("\n** The reference and query FASTA files are required **")
-        sys.exit()
+        sys.exit("\n** The reference and query FASTA files are required **")
 
-    log("RagTag " + get_ragtag_version())
-    log("CMD: ragtag.py scaffold " + " ".join(sys.argv[1:]))
+    log("VERSION", "RagTag " + get_ragtag_version())
+    log("CMD", "ragtag.py scaffold " + " ".join(sys.argv[1:]))
 
     reference_file = os.path.abspath(args.reference)
     query_file = os.path.abspath(args.query)
@@ -300,7 +319,6 @@ def main():
     if not os.path.isfile(query_file):
         raise FileNotFoundError("Could not find file: %s" % query_file)
 
-
     min_ulen = args.f
     keep_small_uniques = not args.remove_small
     merge_dist = args.d
@@ -316,15 +334,16 @@ def main():
     if not os.path.isdir(output_path):
         os.mkdir(output_path)
     output_path = os.path.abspath(output_path) + "/"
+    file_prefix = "ragtag.scaffold"
 
     # Setup a log file for external RagTag scripts
-    ragtag_log = output_path + "ragtag.scaffold.err"
+    ragtag_log = output_path + file_prefix + ".err"
     open(ragtag_log, "w").close()  # Wipe the log file
 
     overwrite_files = args.w
     remove_suffix = not args.u
     if remove_suffix:
-        log("WARNING: Without '-u' invoked, some component/object AGP pairs might share the same ID. Some external programs/databases don't like this. To ensure valid AGP format, use '-u'.")
+        log("WARNING", "Without '-u' invoked, some component/object AGP pairs might share the same ID. Some external programs/databases don't like this. To ensure valid AGP format, use '-u'.")
 
     # Gap options
     min_gap_size = args.g
@@ -339,11 +358,21 @@ def main():
     query_blacklist = set()
     skip_file = args.j
     if skip_file:
-        skip_file = os.path.abspath(args.j)
+        skip_file = os.path.abspath(skip_file)
         with open(skip_file, "r") as f:
             for line in f:
                 query_blacklist.add(line.rstrip())
 
+    skip_no_cat = set()
+    skip_no_cat_file = args.J
+    if skip_no_cat_file:
+        skip_no_cat_file = os.path.abspath(skip_no_cat_file)
+        with open(skip_no_cat_file, "r") as f:
+            for line in f:
+                skip_no_cat.add(line.rstrip())
+
+    query_blacklist = query_blacklist.union(skip_no_cat)
+
     ref_blacklist = set()
     exclude_file = args.e
     if exclude_file:
@@ -355,10 +384,11 @@ def main():
     # Get aligner arguments
     aligner_path = args.aligner
     aligner = aligner_path.split("/")[-1]
-    if aligner.split("/")[-1] not in {'minimap2', 'nucmer'}:
-        raise ValueError("Must specify either 'minimap2' or 'nucmer' (PATHs allowed) with '--aligner'.")
+    if aligner.split("/")[-1] not in {'minimap2', 'unimap', 'nucmer'}:
+        raise ValueError("Must specify either 'minimap2', 'unimap', or 'nucmer' (PATHs allowed) with '--aligner'.")
 
     mm2_params = args.mm2_params
+    unimap_params = args.unimap_params
     nucmer_params = args.nucmer_params
 
     # Mapq filtering params
@@ -366,34 +396,38 @@ def main():
     if aligner == "nucmer":
         min_mapq = 0
 
-    # Add the number of mm2 threads if the mm2 params haven't been overridden.
+    # Add the number of mm2/unimap threads if the mm2 params haven't been overridden.
     if mm2_params == mm2_default:
         mm2_params += " -t " + str(num_threads)
+    if unimap_params == mm2_default:
+        unimap_params += " -t " + str(num_threads)
 
     # Debugging options
     debug_mode = args.debug
-    debug_non_fltrd_file = output_path + "ragtag.scaffolds.debug.unfiltered.paf"
-    debug_fltrd_file = output_path + "ragtag.scaffolds.debug.filtered.paf"
-    debug_merged_file = output_path + "ragtag.scaffolds.debug.merged.paf"
-    debug_query_info_file = output_path + "ragtag.scaffolds.debug.query.info.txt"
+    debug_non_fltrd_file = output_path + file_prefix + ".debug.unfiltered.paf"
+    debug_fltrd_file = output_path + file_prefix + ".debug.filtered.paf"
+    debug_merged_file = output_path + file_prefix + ".debug.merged.paf"
+    debug_query_info_file = output_path + file_prefix + ".debug.query.info.txt"
 
     # Align the query to the reference
-    log("Mapping the query genome to the reference genome")
+    log("INFO", "Mapping the query genome to the reference genome")
     if aligner == "minimap2":
-        al = Minimap2Aligner(reference_file, [query_file], aligner_path, mm2_params, output_path + "query_against_ref", in_overwrite=overwrite_files)
+        al = Minimap2Aligner(reference_file, [query_file], aligner_path, mm2_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
+    elif aligner == "unimap":
+        al = UnimapAligner(reference_file, [query_file], aligner_path, unimap_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
     else:
-        al = NucmerAligner(reference_file, [query_file], aligner_path, nucmer_params, output_path + "query_against_ref", in_overwrite=overwrite_files)
+        al = NucmerAligner(reference_file, [query_file], aligner_path, nucmer_params, output_path + file_prefix + ".asm", in_overwrite=overwrite_files)
     al.run_aligner()
 
     # If alignments are from Nucmer, need to convert from delta to paf
     if aligner == "nucmer":
-        cmd = ["ragtag_delta2paf.py", output_path + "query_against_ref.delta"]
-        run_oae(cmd, output_path + "query_against_ref.paf", ragtag_log)
+        cmd = ["ragtag_delta2paf.py", output_path + file_prefix + ".asm.delta"]
+        run_oae(cmd, output_path + file_prefix + ".asm.paf", ragtag_log)
 
     # Read and organize the alignments
-    log('Reading whole genome alignments')
-    # ctg_alns = dict :: key=query header, value=ContigAlignment object
-    ctg_alns = read_genome_alignments(output_path + "query_against_ref.paf", query_blacklist, ref_blacklist)
+    log("INFO", "Reading whole genome alignments")
+    # ctg_alns: query header -> ContigAlignment object
+    ctg_alns = read_genome_alignments(output_path + file_prefix + ".asm.paf", query_blacklist, ref_blacklist)
 
     # Filter the alignments
     if debug_mode:
@@ -403,7 +437,7 @@ def main():
         open(debug_merged_file, "w").close()
         open(debug_query_info_file, "w").close()
 
-    log("Filtering and merging alignments")
+    log("INFO", "Filtering and merging alignments")
     for i in ctg_alns:
 
         # Write unfiltered alignments
@@ -421,7 +455,7 @@ def main():
                     with open(debug_fltrd_file, "a") as f:
                         f.write(str(ctg_alns[i]))
 
-                ctg_alns[i] = ctg_alns[i].merge_alns(merge_dist=merge_dist)
+                ctg_alns[i] = ctg_alns[i].merge_alns(merge_dist=merge_dist, careful_merge=True)
 
     # Remove query sequences which have no more qualifying alignments
     fltrd_ctg_alns = dict()
@@ -449,9 +483,13 @@ def main():
             ]):
                 fltrd_ctg_alns[i] = ctg_alns[i]
 
+    # Check if any alignments are left
+    if not fltrd_ctg_alns:
+        raise RuntimeError("There are no useful alignments. Check output alignment files.")
+
     # For each reference sequence which has at least one assigned query sequence, get the list of
     # all query sequences assigned to that reference sequence.
-    log("Ordering and orienting query sequences")
+    log("INFO", "Ordering and orienting query sequences")
     mapped_ref_seqs = defaultdict(list)
     for i in fltrd_ctg_alns:
         best_ref = fltrd_ctg_alns[i].best_ref_header
@@ -505,34 +543,34 @@ def main():
             gap_types[i] = ["U" for i in range(len(mapped_ref_seqs[i])-1)]
 
     if infer_gaps:
-        log("%d inferred gap" % g_inferred)
-        log("%d adjacent contig within min distance (%d) of each other" % (g_small, min_gap_size))
-        log("%d inferred gaps exceed length threshold (%d)" % (g_large, max_gap_size))
+        log("INFO", "%d inferred gap" % g_inferred)
+        log("INFO", "%d adjacent contig within min distance (%d) of each other" % (g_small, min_gap_size))
+        log("INFO", "%d inferred gaps exceed length threshold (%d)" % (g_large, max_gap_size))
 
     # Write the scaffolds
-    log("Writing scaffolds")
+    log("INFO", "Writing scaffolds")
 
     # Write the intermediate output file in AGP v2.1 format
-    log("Writing: " + output_path + "ragtag.scaffolds.agp")
-    write_orderings(output_path + "ragtag.scaffolds.agp", output_path + "ragtag.confidence.txt", query_file, mapped_ref_seqs, fltrd_ctg_alns, pad_sizes, gap_types, make_chr0, True, not remove_suffix)
+    log("INFO", "Writing: " + output_path + file_prefix + ".agp")
+    write_orderings(output_path + file_prefix + ".agp", output_path + file_prefix + ".confidence.txt", query_file, mapped_ref_seqs, fltrd_ctg_alns, pad_sizes, gap_types, make_chr0, True, not remove_suffix, skip_no_cat)
 
     # Build a FASTA from the AGP
     cmd = [
-        "ragtag_agp2fasta.py",
-        output_path + "ragtag.scaffolds.agp",
+        "ragtag_agp2fa.py",
+        output_path + file_prefix + ".agp",
         query_file
     ]
-    run_oae(cmd, output_path + "ragtag.scaffolds.fasta", ragtag_log)
+    run_oae(cmd, output_path + file_prefix + ".fasta", ragtag_log)
 
     # Calculate the stats
     cmd = [
         "ragtag_stats.py",
-        output_path + "ragtag.scaffolds.agp",
-        output_path + "ragtag.confidence.txt"
+        output_path + file_prefix + ".agp",
+        output_path + file_prefix + ".confidence.txt"
     ]
-    run_oae(cmd, output_path + "ragtag.scaffolds.stats", ragtag_log)
+    run_oae(cmd, output_path + file_prefix + ".stats", ragtag_log)
 
-    log("Goodbye")
+    log("INFO", "Goodbye")
 
 
 if __name__ == "__main__":
