@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -128,10 +128,10 @@ class Aligner:
                 run_oe(self.compile_command(), self.out_file, self.out_log)
             else:
                 if self.overwrite:
-                    log("overwriting pre-existing file: " + self.out_file)
+                    log("INFO", "Overwriting pre-existing file: " + self.out_file)
                     run_oe(self.compile_command(), self.out_file, self.out_log)
                 else:
-                    log("Retaining pre-existing file: " + self.out_file)
+                    log("INFO", "Retaining pre-existing file: " + self.out_file)
 
 
 class NucmerAligner(Aligner):
@@ -172,10 +172,10 @@ class NucmerAligner(Aligner):
                 run_e(self.compile_command(), self.out_log)
             else:
                 if self.overwrite:
-                    log("overwriting pre-existing file: " + self.out_file)
+                    log("INFO", "Overwriting pre-existing file: " + self.out_file)
                     run_e(self.compile_command(), self.out_log)
                 else:
-                    log("Retaining pre-existing file: " + self.out_file)
+                    log("INFO", "Retaining pre-existing file: " + self.out_file)
 
 
 class Minimap2Aligner(Aligner):
@@ -199,7 +199,45 @@ class Minimap2Aligner(Aligner):
             raise ValueError("Alignments must not be in SAM format (-a).")
 
         if "c" in all_flags:
-            log("WARNING: computing base-alignments (-c) will slow down Minimap2 alignment.")
+            log("WARNING", "Computing base-alignments (-c) will slow down Minimap2 alignment.")
+
+        return True
+
+    def compile_command(self):
+        """
+        Compile the space delimited command to be executed.
+        :return: The command as a string
+        """
+        return [
+            self.aligner,
+            *self.params,
+            self.r_file,
+            *self.q_files
+        ]
+
+
+class UnimapAligner(Aligner):
+    """ The 'Aligner' subclass specific to Unimap alignments in PAF format. """
+
+    def _update_attrs(self):
+        """ Update class attributes for a specific aligner. """
+        self.aligner_exec = "unimap"
+        self.out_file = self.outfile_prefix + ".paf"
+        self.out_log = self.out_file + ".log"
+
+    def params_are_valid(self):
+        """
+        Do a basic check to make sure the unimap parameters are valid.
+        I won't check that every parameter is valid, but will check anything that can
+        cause a problem for RagTag later on.
+        :return: True if the parameters are valid. Raises appropriate errors otherwise
+        """
+        all_flags = "".join([i for i in self.params_string.split(" ") if i.startswith("-")])
+        if "a" in all_flags:
+            raise ValueError("Alignments must not be in SAM format (-a).")
+
+        if "c" in all_flags:
+            log("WARNING", "Computing base-alignments (-c) will slow down Unimap alignment.")
 
         return True
 
