@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -50,8 +50,8 @@ class ContigAlignment:
 
     The set of alignments organized in a ContigAlignment object is used to calculate metrics for the query sequence. For
     example, self.best_ref_header gives the reference sequence which the query sequence most covers. For this reason,
-    the ContigAlignment object is pseudo-immutable. The order of alignments may change (e.g. _sort_by_ref() and
-    _sort_by_query()) but any addition/subtraction of alignments is not allowed. Any such addition/removal functionality
+    the ContigAlignment object is pseudo-immutable. The order of alignments may change (e.g. sort_by_ref() and
+    sort_by_query()) but any addition/subtraction of alignments is not allowed. Any such addition/removal functionality
     (e.g. add_alignment()) must return a new instance of the class.
     """
 
@@ -270,7 +270,7 @@ class ContigAlignment:
 
         self._set_ref_aln_lens()
 
-    def _sort_by_ref(self):
+    def sort_by_ref(self):
         """ Sort the alignments by reference header/position. """
         ref_pos = []
         for i in range(len(self._ref_headers)):
@@ -279,7 +279,7 @@ class ContigAlignment:
 
         self._rearrange_alns(hits)
 
-    def _sort_by_query(self):
+    def sort_by_query(self):
         """ Sort the alignments by query position. """
         q_pos = []
         for i in range(len(self._ref_headers)):
@@ -288,6 +288,54 @@ class ContigAlignment:
 
         self._rearrange_alns(hits)
 
+    @property
+    def num_refs(self):
+        return len(set(self._ref_headers))
+
+    @property
+    def num_alns(self):
+        return len(self._ref_headers)
+
+    @property
+    def query_starts(self):
+        return self._query_starts
+
+    @property
+    def query_ends(self):
+        return self._query_ends
+
+    @property
+    def strands(self):
+        return self._strands
+
+    @property
+    def ref_headers(self):
+        return self._ref_headers
+
+    @property
+    def ref_lens(self):
+        return self._ref_lens
+
+    @property
+    def ref_starts(self):
+        return self._ref_starts
+
+    @property
+    def ref_ends(self):
+        return self._ref_ends
+
+    @property
+    def residue_matches(self):
+        return self._residue_matches
+
+    @property
+    def aln_lens(self):
+        return self._aln_lens
+
+    @property
+    def mapqs(self):
+        return self._mapqs
+
     def add_alignment(self, in_query_start, in_query_end, in_strand, in_reference_header, in_ref_len, in_ref_start, in_ref_end, in_residue_matches, in_aln_len, in_mapq):
         """ Add an alignment for this query sequence. Return a new instance of the class. """
         return ContigAlignment(
@@ -370,13 +418,13 @@ class ContigAlignment:
         hits = [i for i in range(len(self._ref_headers)) if i not in cidx]
         return self._update_alns(hits)
 
-    def merge_alns(self, merge_dist=100000):
+    def merge_alns(self, merge_dist=100000, careful_merge=False):
         """
         Merge adjacent alignments that have the same reference sequence, the same orientation, and are less than
         merge_dist away from each other.
         """
         # Sort the alignments
-        self._sort_by_ref()
+        self.sort_by_ref()
 
         # Make a copy of the alignment info
         query_starts = self._query_starts
@@ -394,11 +442,15 @@ class ContigAlignment:
         i = 0
         j = 1
         while j < len(ref_headers):
-            if all([
-                        ref_headers[i] == ref_headers[j],
-                        strands[i] == strands[j],
-                        ref_starts[j] - ref_ends[i] <= merge_dist
-            ]):
+            conditions = [
+                ref_headers[i] == ref_headers[j],
+                strands[i] == strands[j],
+                ref_starts[j] - ref_ends[i] <= merge_dist
+            ]
+            if careful_merge:
+                conditions.append(query_starts[j] - query_ends[i] <= merge_dist)
+
+            if all(conditions):
                 # Merge the alignments in place of the first alignment
                 query_starts[i] = min(query_starts[i], query_starts[j])
                 query_ends[i] = max(query_ends[i], query_ends[j])
@@ -455,7 +507,7 @@ class ContigAlignment:
         same reference (intra) and one where the consecutive alignments aligned to different
         references (inter).
         """
-        self._sort_by_query()
+        self.sort_by_query()
         intra_candidates = []
         inter_candidates = []
 
@@ -469,3 +521,61 @@ class ContigAlignment:
 
         return intra_candidates, inter_candidates
 
+    def keep_terminals(self, max_term_dist):
+        """
+        remove any alignments that aren't within max_term_dist of a reference terminus
+        :param max_term_dist: maximum distance from a reference terminus
+        """
+        if max_term_dist <= 0:
+            raise ValueError("max_term_dist must be a positive nonzero number.")
+
+        if max_term_dist < 1:
+            hits = [i for i in range(self.num_alns) if self.ref_starts[i] <= (max_term_dist*self.ref_lens[i]) or (self.ref_lens[i] - self.ref_ends[i]) <= (max_term_dist*self.ref_lens[i])]
+        else:
+            hits = [i for i in range(self.num_alns) if self.ref_starts[i] <= max_term_dist or (self.ref_lens[i] - self.ref_ends[i]) <= max_term_dist]
+        return self._update_alns(hits)
+
+    def ref_start_end(self, aln_idx, max_term_dist):
+        """
+        :param aln_idx: Index of the alignment
+        :param max_term_dist: maximum distance from a reference terminus
+        """
+        if max_term_dist <= 0:
+            raise ValueError("max_term_dist must be a positive nonzero number.")
+
+        if max_term_dist < 1:
+            max_term_dist = max_term_dist * self.ref_lens[aln_idx]
+
+        return self.ref_starts[aln_idx] <= max_term_dist, (self.ref_lens[aln_idx] - self.ref_ends[aln_idx]) <= max_term_dist
+
+    def query_start_end(self, aln_idx, max_term_dist):
+        """
+        :param aln_idx: Index of the alignment
+        :param max_term_dist: maximum distance from a query terminus
+        """
+        if max_term_dist <= 0:
+            raise ValueError("max_term_dist must be a positive nonzero number.")
+
+        if max_term_dist < 1:
+            max_term_dist = max_term_dist * self.ref_lens[aln_idx]
+
+        return self.query_starts[aln_idx] <= max_term_dist, (self.query_len - self.query_ends[aln_idx]) <= max_term_dist
+
+    def has_internal_ref_cuttings(self, max_term_dist):
+        """
+        This is when a reference sequence is well contained within a query sequence, yet there are parts of the
+        reference sequence that don't align. Should be used after filtering an merging.
+        :return:
+        """
+        self.sort_by_query()
+        for i in range(self.num_alns):
+
+            # For each reference/query alignment terminus, determine if it is close to the sequence terminus
+            ref_left_end, ref_right_end = self.ref_start_end(i, max_term_dist)
+            query_left_end, query_right_end = self.query_start_end(i, max_term_dist)
+
+            if not query_left_end and not query_right_end:
+                if not ref_left_end or not ref_right_end:
+                    return True
+
+        return False
