@@ -3,7 +3,7 @@
 """
 MIT License
 
-Copyright (c) 2020 Michael Alonge <malonge11@gmail.com>
+Copyright (c) 2021 Michael Alonge <malonge11@gmail.com>
 
 Permission is hereby granted, free of charge, to any person obtaining a copy
 of this software and associated documentation files (the "Software"), to deal
@@ -30,9 +30,11 @@ import itertools
 import numbers
 
 import pysam
+import numpy as np
 import networkx as nx
 
 from ragtag_utilities.AGPFile import AGPFile
+from ragtag_utilities.utilities import get_ragtag_version
 
 
 """
@@ -207,7 +209,7 @@ class ScaffoldGraphBase:
             raise ValueError("Nodes must end with either '_b' or '_e'.")
 
         if n[:-2] not in self.component_lens:
-            raise ValueError("{} was not found in {}.".format(n[:-2], self.components_fasta_fname))
+            raise ValueError("{} was not found in {}".format(n[:-2], self.components_fasta_fname))
 
         self.graph.add_node(n, **kwargs)
 
@@ -259,14 +261,20 @@ class ScaffoldGraphBase:
 
     def write_gml(self, f):
         """
-        Write the graph in GEXF format.
+        Write the graph in GML format.
         :param f: Write the graph to this file handle.
         """
-        nx.readwrite.gml.write_gml(self.graph, f)
+        G = self.graph.copy()
+
+        # networkx doesn't like writing non-string attributes to GML
+        for u, v in G.edges:
+            for key in G[u][v]:
+                G[u][v][key] = str(G[u][v][key])
+        nx.readwrite.gml.write_gml(G, f)
 
     def connect_and_write_gml(self, f):
         """
-        Connect scaffold edges, then write the graph in GEXF format
+        Connect scaffold edges, then write the graph in GML format
         :param f: Write the graph to this file handle.
         """
         G = self.graph.copy()
@@ -274,6 +282,10 @@ class ScaffoldGraphBase:
         for node in node_base_set:
             G.add_edge(node + "_b", node + "_e")
 
+        # networkx doesn't like writing non-string attributes to GML
+        for u, v in G.edges:
+            for key in G[u][v]:
+                G[u][v][key] = str(G[u][v][key])
         nx.readwrite.gml.write_gml(G, f)
 
     def filter_by_seq_len(self, min_len):
@@ -316,6 +328,19 @@ class ScaffoldGraph(ScaffoldGraphBase):
 
         self.graph = G
 
+    def filter_non_one(self):
+        """ Remove edges where the weight is not equal to one. """
+        G = nx.Graph()
+
+        for u, v in self.edges:
+            if self.graph[u][v]["weight"] == 1:
+                # Add the nodes first in case they have data
+                G.add_node(u, **self.nodes(data=True)[u])
+                G.add_node(v, **self.nodes(data=True)[v])
+                G.add_edge(u, v, **self.graph[u][v])
+
+        self.graph = G
+
     def get_max_weight_matching(self):
         return nx.max_weight_matching(G=self.graph)
 
@@ -581,6 +606,8 @@ class AGPMultiScaffoldGraph(MultiScaffoldGraph):
             for ap in self._get_assembly_points(agp, weight):
                 for u, v, w in ap.get_realizations():
                     if u[:-2] not in exclusion_set and v[:-2] not in exclusion_set:
+                        u_base, v_base = u[:-2], v[:-2]
+                        u_len = self.get_component_len(u_base)
                         self.add_edge(
                             u,
                             v,
@@ -590,5 +617,349 @@ class AGPMultiScaffoldGraph(MultiScaffoldGraph):
                             gap_size=ap.gap_size,
                             gap_type=ap.gap_type,
                             linkage=ap.linkage,
-                            linkage_evidence=ap.linkage_evidence
+                            linkage_evidence=ap.linkage_evidence,
+                            seqs=[u_base, v_base],
+                            pos=[u_len, 0]
                         )
+
+
+class Alignment:
+    """
+    An object representing query a sequence/gap joining two target sequences.
+
+      target_from --------- target "from" scaffold graph node
+      target_to ----------- target "to" scaffold graph node
+      query --------------- name of the sequence used to join "from" and "to". ignored if gap
+      query_len ----------- length of the query sequence
+      my_query_end -------- the 0-based position in the query that corresponds to the end of "from"
+      their_query_start --- the 0-based position in the query that corresponds to the beginning of "to"
+      strand -------------- the strand of the query sequence
+      is_gap -------------- boolean indicating a gap or a sequence
+    """
+
+    def __init__(self, target_from, target_to, query, query_len, my_query_end, their_query_start, strand, is_gap=False):
+        """
+
+        :param target_from:
+        :param target_to:
+        :param query:
+        :param query_len:
+        :param my_query_end:
+        :param their_query_start:
+        :param strand: 0, 1 for forward, reverse
+        :param is_gap:
+        """
+        self.target_from = target_from
+        self.target_to = target_to
+        self.query = query
+        self.query_len = query_len
+        self.my_query_end = my_query_end  # 0-based query start coordinate
+        self.their_query_start = their_query_start  # 0-based query end coordinate
+        self.strand = strand  # 0, 1 for forward, reverse
+
+        self.is_gap = is_gap
+        self.query_span = self.their_query_start - self.my_query_end
+
+    def __str__(self):
+        return "::".join([
+            self.target_from,
+            self.target_to,
+            self.query
+        ])
+
+    def reverse(self):
+        return Alignment(
+            self.target_to,
+            self.target_from,
+            self.query,
+            self.query_len,
+            self.my_query_end,
+            self.their_query_start,
+            1 - self.strand,
+            is_gap=self.is_gap
+        )
+
+
+class PatchScaffoldGraph:
+    """
+    A directed scaffold graph useful for continuous scaffolding
+
+    Terms:
+      Target assembly - a genome assembly that we wish to scaffold
+      Target sequence - an individual sequence in a target assembly
+      Query sequence - a sequence, independent from the target assembly, used to scaffold the target assembly
+      Adjacency - a pair of termini between two distinct target sequences suggested to be joined
+
+    The object keeps track of a directed graph. Nodes in the graph are target sequence termini
+    (target sequence + "_b" or "_e" representing the beginning or end of the sequence, respectively). Edges
+    connecting these nodes represent an adjacency, and the necessary information must be encoded in an Alignment
+    object.
+
+    This class defines a method to produce an AGP file from the scaffold graph, assuming that the edges
+    form a matching.
+    """
+
+    def __init__(self, components_fn):
+        self.graph = nx.DiGraph()
+
+        self.components_fn = components_fn  # name of FASTA file with all relevant target/query sequences
+        self.component_lens = dict()  # component name -> length
+        self._set_component_lens()
+
+    def __getitem__(self, e):
+        return self.graph[e]
+
+    def _set_component_lens(self):
+        fai = pysam.FastaFile(self.components_fn)
+
+        for i in fai.references:
+            self.component_lens[i] = fai.get_reference_length(i)
+
+    @property
+    def nodes(self, **kwargs):
+        return self.graph.nodes(**kwargs)
+
+    @property
+    def edges(self):
+        return self.graph.edges
+
+    def has_edge(self, u, v):
+        return self.graph.has_edge(u, v)
+
+    def neighbors(self, n):
+        return self.graph.neighbors(n)
+
+    def add_edge(self, u, v, aln):
+        """
+        :param u: from
+        :param v: to
+        :param aln: Alignment object
+        """
+        if not isinstance(aln, Alignment):
+            raise TypeError("aln must be an Alignment object.")
+
+        if u != aln.target_from or v != aln.target_to:
+            raise RuntimeError("Scaffold graph nodes don't match Alignment nodes.")
+
+        if self.has_edge(u, v):
+            self.graph[u][v]["weight"] += 1
+            self.graph[u][v]["alignment"] = aln
+
+            self.graph[v][u]["weight"] += 1
+            self.graph[v][u]["alignment"] = aln.reverse()
+        else:
+            self.graph.add_edge(u, v, weight=1, alignment=aln)
+            self.graph.add_edge(v, u, weight=1, alignment=aln.reverse())
+
+    def remove_edge(self, u, v):
+        self.graph.remove_edge(u, v)
+
+    def write_gml(self, f):
+        G = nx.DiGraph()
+        for u, v in self.edges:
+            G.add_edge(
+                u,
+                v,
+                target_from=self.graph[u][v]["alignment"].target_from,
+                target_to=self.graph[u][v]["alignment"].target_to,
+                query=self.graph[u][v]["alignment"].query,
+                query_len=str(self.graph[u][v]["alignment"].query_len),
+                my_query_end=str(self.graph[u][v]["alignment"].my_query_end),
+                their_query_start=str(self.graph[u][v]["alignment"].their_query_start),
+                strand=str(self.graph[u][v]["alignment"].strand),
+                is_gap=str(self.graph[u][v]["alignment"].is_gap)
+            )
+
+            G.add_edge(
+                v,
+                u,
+                target_from=self.graph[v][u]["alignment"].target_from,
+                target_to=self.graph[v][u]["alignment"].target_to,
+                query=self.graph[v][u]["alignment"].query,
+                query_len=str(self.graph[v][u]["alignment"].query_len),
+                my_query_end=str(self.graph[v][u]["alignment"].my_query_end),
+                their_query_start=str(self.graph[v][u]["alignment"].their_query_start),
+                strand=str(self.graph[v][u]["alignment"].strand),
+                is_gap=str(self.graph[v][u]["alignment"].is_gap)
+            )
+        nx.readwrite.gml.write_gml(G, f)
+
+    def remove_heavier_than(self, w):
+        """
+        Remove edges with weight > w
+        :param w: maximum edge weight
+        """
+        G = nx.DiGraph()
+        for u, v in self.edges:
+            if self.graph[u][v]["weight"] <= w:
+                G.add_edge(u, v, weight=self.graph[u][v]["weight"], alignment=self.graph[u][v]["alignment"])
+        self.graph = G
+
+    def max_weight_matching(self):
+        """
+        Return a new PatchScaffoldGraph that is a matching
+        :return: Matching PatchScaffoldGraph
+        """
+        undirected_sg = nx.Graph()
+        for u, v in self.edges:
+            undirected_sg.add_edge(u, v, weight=self.graph[u][v]["weight"])
+        matching = nx.max_weight_matching(G=undirected_sg)
+
+        # Remove any cycles that would result from adding intra-sequence edges
+        # Make a new graph
+        cover_graph = nx.Graph()
+        cover_graph.add_nodes_from(undirected_sg.nodes(data=True))
+
+        # Add edges connecting contig ends
+        node_base_set = set([i[:-2] for i in list(cover_graph.nodes)])
+        for node in node_base_set:
+            cover_graph.add_edge(node + "_b", node + "_e", weight=np.inf)
+
+        # Add the edges that form the matching
+        for u, v in matching:
+            cover_graph.add_edge(u, v, **undirected_sg[u][v])
+
+        # Remove any potential cycles
+        edges_to_delete = []
+        for cc in nx.connected_components(G=cover_graph):
+            cc = cover_graph.subgraph(cc).copy()
+            if cc.number_of_nodes() == cc.number_of_edges():
+                assembly_edges = cc.edges(data=True)
+                edge_to_delete = min(assembly_edges, key=lambda entry: entry[2]["weight"])
+                edges_to_delete.append((edge_to_delete[0], edge_to_delete[1]))
+
+        # Add the edges that form the matching into a new PatchScaffoldGraph
+        new_psg = PatchScaffoldGraph(self.components_fn)
+        for u, v in matching:
+            if (u, v) not in edges_to_delete and (v, u) not in edges_to_delete:
+                new_psg.add_edge(u, v, self.graph[u][v]["alignment"])
+                new_psg.add_edge(v, u, self.graph[v][u]["alignment"])
+
+        return new_psg
+
+    def write_agp(self, agp_fn, ref_fn, add_suffix_to_unplaced=False):
+        """
+        Write the AGP file implied by the scaffold graph
+        :param agp_fn: AGP file name
+        :param ref_fn: reference FASTA file name
+        :param add_suffix_to_unplaced: add "_RagTag" to unscaffolded sequences
+        """
+        used_components = set()
+        used_edges = set()
+        obj_header_idx = -1
+
+        agp = AGPFile(agp_fn, "w")
+        agp.add_pragma()
+        agp.add_comment("# AGP created by RagTag {}".format(get_ragtag_version()))
+
+        while True:
+            # Find a starting node
+            from_node = None
+            to_node = None
+            cur_ref = None
+            for u, v in sorted(self.edges):
+                if (u, v) not in used_edges:
+                    u_base = u[:-2]
+
+                    u_degree = 0
+                    if u_base + "_b" in self.nodes:
+                        u_degree += self.graph.degree[u_base + "_b"]
+                    if u_base + "_e" in self.nodes:
+                        u_degree += self.graph.degree[u_base + "_e"]
+
+                    assert u_degree in {2, 4}
+
+                    # Check if we have found a starting target sequence
+                    if u_degree == 2:
+                        cur_ref = u_base
+                        from_node = u
+                        to_node = v
+                        used_edges.add((u, v))
+                        used_edges.add((v, u))
+                        break
+
+            # If we haven't found a new starting target sequence, we are done
+            if from_node is None:
+                break
+
+            # Initialize this object
+            obj_header_idx += 1
+            obj_header = "scf" + "{0:08}".format(obj_header_idx)
+            obj_pos = 0
+            obj_pid = 1
+
+            # Process the first target sequence
+            cur_ref_len = self.component_lens[cur_ref]
+            cur_ref_strand = "+"
+            if from_node.endswith("_b"):
+                cur_ref_strand = "-"
+            agp.add_seq_line(obj_header, obj_pos+1, obj_pos+cur_ref_len, obj_pid, "W", cur_ref, 1, cur_ref_len, cur_ref_strand)
+            obj_pos += cur_ref_len
+            obj_pid += 1
+            used_components.add(cur_ref)
+
+            # Process the remaining sequences.
+            next_edge_exists = True
+            while next_edge_exists:
+                # Process the patch
+                patch_aln = self.graph[from_node][to_node]["alignment"]
+                patch_query = patch_aln.query
+                patch_strand = "+"
+                if patch_aln.strand:
+                    patch_strand = "-"
+
+                patch_len = patch_aln.their_query_start - patch_aln.my_query_end
+                if patch_len > 0:
+                    if patch_aln.is_gap:
+                        agp.add_gap_line(obj_header, obj_pos+1, obj_pos+patch_len, obj_pid, "N", patch_len, "scaffold", "yes", "align_genus")
+                    else:
+                        agp.add_seq_line(obj_header, obj_pos+1, obj_pos+patch_len, obj_pid, "W", patch_query, patch_aln.my_query_end+1, patch_aln.their_query_start, patch_strand)
+                        used_components.add(patch_query)
+                    obj_pos += patch_len
+                    obj_pid += 1
+
+                # Next, process the reference sequence
+                comp_start = min(0, patch_len)
+                cur_ref = to_node[:-2]
+                cur_ref_len = self.component_lens[cur_ref]
+                cur_ref_strand = "+"
+                if to_node.endswith("_e"):
+                    cur_ref_strand = "-"
+                agp.add_seq_line(obj_header, obj_pos+1, obj_pos+(cur_ref_len + comp_start), obj_pid, "W", cur_ref, 1+(-1*comp_start), cur_ref_len, cur_ref_strand)
+                obj_pos += cur_ref_len + comp_start
+                obj_pid += 1
+                used_components.add(cur_ref)
+
+                # Look for the next edge
+                from_node = to_node[:-2] + "_b"
+                if to_node.endswith("_b"):
+                    from_node = to_node[:-2] + "_e"
+
+                if from_node in self.graph.nodes:
+                    next_nodes = set(self.graph[from_node])
+                    assert len(next_nodes) == 1
+                    to_node = next_nodes.pop()
+                    used_edges.add((from_node, to_node))
+                    used_edges.add((to_node, from_node))
+                else:
+                    next_edge_exists = False
+
+        # Write unplaced reference sequences
+        fai = pysam.FastaFile(ref_fn)
+        all_ref_seqs = set(fai.references)
+        fai.close()
+        remaining_components = all_ref_seqs - used_components
+        for c in sorted(remaining_components):
+            agp.add_seq_line(
+                c + "_RagTag" * add_suffix_to_unplaced,
+                "1",
+                str(self.component_lens[c]),
+                "1",
+                "W",
+                c,
+                "1",
+                str(self.component_lens[c]),
+                "+"
+            )
+
+        agp.write()
